#!/usr/bin/python

# ----------------------------------------------------------------------------
#
#  Copyright (C) 2015 Fons Adriaensen <fons@linuxaudio.org>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


import socket
from tcpconn import Tcpconn
from alsacontrol import AlsaControl
from jacktools.jackkmeter import JackKmeter

alsactl = AlsaControl ()
if alsactl.open ("hw:MODDUO"):
    print ("Can't open ALSA control device.")
    exit (1)

kmeter = JackKmeter (4, 'Kmeter')
if kmeter.get_levels ()[0] < 0:
    print ("Can't open Jack client.")
    exit (1)
kmeter.connect_input (0, 'system:capture_1')
kmeter.connect_input (1, 'mod-host:monitor-in_1')
kmeter.connect_input (2, 'system:capture_2')
kmeter.connect_input (3, 'mod-host:monitor-in_2')


def mixer (M):
    print (M)
    alsactl.cset (M [1], M [2])


def levels (M):
    K = kmeter.get_levels ()
    if K [0] < 0:
        exit (1)
    else:
        R = K [1]
        P = K [2]
        L = [(R [i], P [i]) for i in range (4)]
    tcph.send_object (['levels', L])


actions = { 'mixer':mixer, 'levels':levels }
servaddr = ('192.168.51.1', 4000)


while True:
    sock = socket.socket (socket.AF_INET, socket.SOCK_STREAM)
    sock.setsockopt (socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    sock.bind (servaddr)
    sock.listen (1)
    print ("Waiting for connection...")
    sock = sock.accept ()[0]
    print ("Client connected.")
    tcph = Tcpconn (sock)
    try:
        while True:
            X = tcph.recv_object ()
            actions [X [0]](X)
    except:
        del tcph
        del sock

